#ifndef UTILS_HH_
#define UTILS_HH_
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <vector>

inline std::string removeLeadingBlanks(std::string s)
{
  int i=0;
  while(s[i]==' '){
    i++;
  }
  std::string newstring;
  for(int j=i;j<s.size();j++) newstring += s[j];
  return newstring;
}

inline std::string deblank(std::string s)
{
  int beg=0;
  while(s[beg]==' '){
    beg++;
  }
  int end=s.size()-1;
  while(s[end]==' '){
    end--;
  }
  std::string newstring;
  for(int j=beg;j<=end;j++) newstring += s[j];
  return newstring;
}

inline std::string charToBinary(char c)
{
  std::stringstream ss;
  int num=c;
  if (num<0) num+=256;
  int factor = 128;
  for(int i=0;i<8;i++){
    ss << num/factor;
    num -= num/factor * factor;
    factor /= 2;
  }
  return ss.str();
}

inline std::string binaryToString(std::vector<bool> v)
{
  int size = v.size();
  int nbytes = (size-1)/8 + 1;
  std::string output;
  for(int byte=0;byte<nbytes;byte++){
    int c=0;
    for(int bit=0;bit<8;bit++){
      int pt = byte*8+bit;
      if(v[pt]) c += int(pow(2,7-bit));
    }
    //    std::cerr << int(c) << std::endl;
    output += char(c);
  }
  return output;
}

inline unsigned int * binaryToInt(std::vector<bool> v)
{
  int size = v.size();
  int nbytes = (size-1)/8 + 1;
  unsigned int *output = new unsigned int[nbytes];
  for(int byte=0;byte<nbytes;byte++){
    int c=0;
    for(int bit=0;bit<8;bit++){
      int pt = byte*8+bit;
      if(v[pt]) c += (unsigned int)pow(2,7-bit);
    }
    //    std::cerr << c << std::endl;
    output[byte] = c;
  }
  return output;
}

inline std::string getPSRFITSdefn()
{
  std::string defn = getenv("PSRFITSDEFN");
  return defn;
}

double dmsToDec(std::string input);  // in Utils.cc

inline std::string getIAUName(double ra, double dec)
{
  /**
   * string getIAUName(double, double)
   *  both ra and dec are assumed to be in degrees.
   *  returns name of the form J1234m4321 or J1234p0123
   *  (note -- assumes J2000.0)
   */

  double raHrs = ra / 15.;
  int h = int(raHrs);
  int m = (int)(fmod(raHrs,1.)*60.);
  int s = (int)(fmod(raHrs,1./60.)*3600.);
  std::stringstream ss(std::stringstream::out);
  ss.setf(std::ios::showpoint);
  ss.setf(std::ios::fixed);
  ss << "J"; 
  ss<<std::setw(2)<<std::setfill('0')<<h;
  ss<<std::setw(2)<<std::setfill('0')<<m;
  ss<<std::setw(2)<<std::setfill('0')<<s;
  int sign = int( dec / fabs(dec) );
  double d = dec / sign;
  h = int(d);
  m = (int)(fmod(d,1.)*60.);
  s = (int)(fmod(d,1./60.)*3600.);
  if(sign==1) ss<<"p"; else ss<<"m";
  ss<<std::setw(2)<<std::setfill('0')<<h;
  ss.unsetf(std::ios::showpos);
  ss<<std::setw(2)<<std::setfill('0')<<m;
  ss<<std::setw(2)<<std::setfill('0')<<s;
  return ss.str();
}

#endif /*UTILS_HH_*/
